From 1037a25cc693f6da7d3d3be7919f45bd1e337f00 Mon Sep 17 00:00:00 2001
From: Larry Shaffer <larrys@dakotacarto.com>
Date: Sun, 16 Dec 2012 20:02:07 -0700
Subject: [PATCH] Keep attribute editor QComboBox/QlineEdit synchronization
 from affecting editing

- Cursor no longer jumps to end of text after each edit (on Mac)
---
 src/gui/qgsattributeeditor.cpp | 17 ++++++++++++-----
 src/gui/qgsattributeeditor.h   | 13 +++++++++++++
 2 files changed, 25 insertions(+), 5 deletions(-)

diff --git a/src/gui/qgsattributeeditor.cpp b/src/gui/qgsattributeeditor.cpp
index 8b74f94..507a225 100644
--- a/src/gui/qgsattributeeditor.cpp
+++ b/src/gui/qgsattributeeditor.cpp
@@ -494,8 +494,10 @@ QWidget *QgsAttributeEditor::createAttributeEditor( QWidget *parent, QWidget *ed
         myWidget = pte;
       }
 
+      QLineEdit* cble = 0;
       if ( cb )
       {
+        cble = cb->lineEdit();
         myWidget = cb;
       }
 
@@ -516,15 +518,17 @@ QWidget *QgsAttributeEditor::createAttributeEditor( QWidget *parent, QWidget *ed
           relay = new QgsStringRelay( myWidget );
         }
 
-        if ( cb && cb->isEditable() )
+        if ( cb && cb->isEditable() && cble )
         {
-          synchronized =  connect( relay, SIGNAL( textChanged( QString ) ), myWidget, SLOT( setEditText( QString ) ) );
-          synchronized &= connect( myWidget, SIGNAL( editTextChanged( QString ) ), relay, SLOT( changeText( QString ) ) );
+          synchronized =  connect( relay, SIGNAL( textChanged( QString ) ), cb, SLOT( setEditText( QString ) ) );
+          synchronized &= connect( cble, SIGNAL( textChanged( QString ) ), relay, SLOT( storeText( QString ) ) );
+          synchronized &= connect( cble, SIGNAL( editingFinished() ), relay, SLOT( sendStoredText() ) );
         }
         else
         {
           synchronized =  connect( relay, SIGNAL( textChanged( QString ) ), myWidget, SLOT( setText( QString ) ) );
-          synchronized &= connect( myWidget, SIGNAL( textChanged( QString ) ), relay, SLOT( changeText( QString ) ) );
+          synchronized &= connect( myWidget, SIGNAL( textChanged( QString ) ), relay, SLOT( storeText( QString ) ) );
+          synchronized &= connect( myWidget, SIGNAL( editingFinished() ), relay, SLOT( sendStoredText() ) );
         }
 
         if ( !cb || cb->isEditable() )
@@ -888,9 +892,10 @@ bool QgsAttributeEditor::setValue( QWidget *editor, QgsVectorLayer *vl, int idx,
     default:
     {
       QLineEdit *le = qobject_cast<QLineEdit *>( editor );
+      QComboBox *cb = qobject_cast<QComboBox *>( editor );
       QTextEdit *te = qobject_cast<QTextEdit *>( editor );
       QPlainTextEdit *pte = qobject_cast<QPlainTextEdit *>( editor );
-      if ( !le && !te && !pte )
+      if ( !le && ! cb && !te && !pte )
         return false;
 
       QString text;
@@ -910,6 +915,8 @@ bool QgsAttributeEditor::setValue( QWidget *editor, QgsVectorLayer *vl, int idx,
 
       if ( le )
         le->setText( text );
+      if ( cb && cb->isEditable() )
+        cb->setEditText( text );
       if ( te )
         te->setHtml( text );
       if ( pte )
diff --git a/src/gui/qgsattributeeditor.h b/src/gui/qgsattributeeditor.h
index b062ff2..f279370 100644
--- a/src/gui/qgsattributeeditor.h
+++ b/src/gui/qgsattributeeditor.h
@@ -95,8 +95,21 @@ class QgsStringRelay : public QObject
       emit textChanged( str );
     }
 
+    void storeText( QString str )
+    {
+      mStoredText = str;
+    }
+
+    void sendStoredText()
+    {
+      emit textChanged( mStoredText );
+    }
+
   signals:
     void textChanged( QString );
+
+  private:
+    QString mStoredText;
 };
 
 #endif
-- 
1.7.11.1

